import org.w3c.tools.resources.ResourceReference;

import java.lang.ref.WeakReference;
import java.util.*;
import java.util.Map.Entry;

aspect LORAspectWithMemoryManagement {
    
    public class WeakPair
    {
        public WeakPair( Object o1, Object o2 ) { this.o1 = new WeakReference(o1); this.o2 = new WeakReference(o2); }
        public int hashCode() {
            int code = 0;
            code += (o1.get()!=null) ? o1.get().hashCode() : 0;
            code += (o2.get()!=null) ? o2.get().hashCode() : 0;
            return code;
        }
        public boolean equals( Object other ) {
            if(other==this)
                return true;
            if( other instanceof WeakPair ) {
                WeakPair p = (WeakPair) other;
                //compare on o1
                Object to1 = o1.get();
                Object oo1 = ((WeakPair)other).o1.get();
                if(to1==null) {
                    if(oo1!=null) {
                        return false;
                    }
                    //else both null
                } else {
                    if(oo1==null) {
                        return false;
                    } else {
                        if(!to1.equals(oo1)) {
                            return false;
                        }
                    }
                }
                //compare on o2
                Object to2 = o2.get();
                Object oo2 = ((WeakPair)other).o2.get();
                if(to2==null) {
                    if(oo2!=null) {
                        return false;
                    }
                    //else both null
                } else {
                    if(oo2==null) {
                        return false;
                    } else {
                        if(!to2.equals(oo2)) {
                            return false;
                        }
                    }
                }                
                return true;
            } else return false;
        }
        public String toString() {
            return "WeakPair ("+o1.get()+","+o2.get()+")";
        }
        public Object getO1() { return o1.get(); }
        public Object getO2() { return o2.get(); }

        protected WeakReference o1;
        protected WeakReference o2;
        
        public boolean collectable() {
            return this.o1.get()==null || this.o2.get()==null;
        }
    }

	pointcut lock(Thread t, org.w3c.tools.resources.ResourceReference r) :
		Logger.lock() && target(r) && let(t,Thread.currentThread());

	pointcut unlock(Thread t,org.w3c.tools.resources.ResourceReference r) :
		Logger.unlock() && target(r) && let(t,Thread.currentThread());

    WeakHashMap threadToRes = new WeakHashMap(); 
    WeakHashMap threadToPairs = new WeakHashMap();
    
	synchronized before(Thread t, org.w3c.tools.resources.ResourceReference r): lock(t,r) {
	    Set res = (Set) threadToRes.get(t);
        if(res==null) {
            res = new WeakIdentityHashSet();
            threadToRes.put(t,res);
        }
        
        for (Iterator iter = res.iterator(); iter.hasNext();) {
            ResourceReference re = (ResourceReference) iter.next();
            
            Set pairs = (Set) threadToPairs.get(t);
            if(pairs==null) {
                pairs = new HashSet();
                threadToPairs.put(t,pairs);
            }
            pairs.add(new WeakPair(re,r));
            
            WeakPair inverse = new WeakPair(r,re);
            for (Iterator threadIter = threadToPairs.entrySet().iterator(); threadIter.hasNext();) {
                Entry entry = (Entry) threadIter.next();
                if(((Set)entry.getValue()).contains(inverse) && entry.getKey()!=t) {
                    System.out.println("Lock order reversal detected!!!");
                }
            }
        }
        res.add(r);
        
        //clean up
        removeCollectablePairs(t);
    }

    synchronized before(Thread t, org.w3c.tools.resources.ResourceReference r): unlock(t,r) {
        Set res = (Set) threadToRes.get(t);
        if(res!=null)        
            res.remove(r);        
        //clean up
        removeCollectablePairs(t);
    }
    
    private void removeCollectablePairs(Thread t) {
        Set pairs = (Set) threadToPairs.get(t);
        if(pairs==null)
            return;
        
        for (Iterator iter = pairs.iterator(); iter.hasNext();) {
            WeakPair pair = (WeakPair) iter.next();
            if(pair.collectable()) {
                iter.remove();
            }            
        }
    }
}
