/*
 * @(#)Animator.java
 *
 * Project:		JHotdraw - a GUI framework for technical drawings
 *				http://www.jhotdraw.org
 *				http://jhotdraw.sourceforge.net
 * Copyright:	 by the original author(s) and all contributors
 * License:		Lesser GNU Public License (LGPL)
 *				http://www.opensource.org/licenses/lgpl-license.html
 */

package org.jhotdraw.samples.javadraw;

import org.jhotdraw.framework.*;
import org.jhotdraw.jtil.Animatable;

/**
 * @version <$CURRENT_VERSION$>
 */
public  class Animator extends Thread {

	private DrawingView     fView;
	private Animatable      fAnimatable;

	private volatile boolean             fIsRunning;
	private static final int    DELAY = 1000 / 16;

	public Animator(Animatable animatable, DrawingView view) {
		super("Animator");
		fView = view;
		fAnimatable = animatable;
	}

	public void start() {
		super.start();
		fIsRunning = true;
	}

	public void end() {
		fIsRunning = false;
	}

	public void run() {
    	Runtime rt = java.lang.Runtime.getRuntime();
    	long start = System.currentTimeMillis();
    	if(JavaDrawApp.USE_GARBAGE_COLLECTION_EACH_ITERATION) {
	    	for (int i=0; i<=100000; i++) {
			    if(i % 100 == 0) {
			    	rt.gc();
			    	System.out.println(i + "\t" + System.currentTimeMillis() + "\t" + (rt.totalMemory() - rt.freeMemory()));
			    }
	            fView.freezeView();
	            fAnimatable.animationStep();
	            fView.checkDamage();
	            fView.unfreezeView();
	        }
    	} else {
	    	for (int i=0; i<=100000; i++) {
			    if(i % 100 == 0) {
			    	System.out.println(i + "\t" + System.currentTimeMillis() + "\t" + (rt.totalMemory() - rt.freeMemory()));
			    }
	            fView.freezeView();
	            fAnimatable.animationStep();
	            fView.checkDamage();
	            fView.unfreezeView();
	        }
    	}
    	long end = System.currentTimeMillis();
    	System.err.println("animation took: "+(end - start));
    	rt.gc();
	// System.err.println("max - free:" + (rt.maxMemory() - rt.freeMemory()));
        System.out.println("memory used: "+ (rt.totalMemory() - rt.freeMemory()));
        if(JavaDrawApp.EXIT_AFTER_ANIMATION) System.exit(0);
    }
}

