/*
 * abc - The AspectBench Compiler Copyright (C) 2005
 * 
 * This compiler is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by the
 * Free Software Foundation; either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This compiler is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this compiler, in the file LESSER-GPL; if not, write to the Free
 * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */
package observdrawchange;

import org.jhotdraw.standard.*;
import org.jhotdraw.framework.*;

/**
 * @author Neil Ongkingco
 *  
 */
public aspect DrawChangeNotify {
    
    private boolean debug = false;
    private void debPrint(String s) {
        if (debug) {
            System.out.println(s);
        }
    }
    
    //--------------------------------------------------------------------
    //Pointcut definitions
    pointcut addDrawListener(
            org.jhotdraw.framework.DrawingChangeListener listener,
            org.jhotdraw.standard.StandardDrawing object):
        execution (* org.jhotdraw.standard.StandardDrawing+.addDrawingChangeListener(org.jhotdraw.framework.DrawingChangeListener+)) 
        && this(object) && args(listener);

    pointcut removeDrawListener(
            org.jhotdraw.framework.DrawingChangeListener listener,
            org.jhotdraw.standard.StandardDrawing object):
        execution (* org.jhotdraw.standard.StandardDrawing+.removeDrawingChangeListener(org.jhotdraw.framework.DrawingChangeListener+)) 
        && this(object) && args(listener);

    pointcut figureInvalidated(org.jhotdraw.framework.FigureChangeEvent event,
            org.jhotdraw.standard.StandardDrawing object) :
        execution (* org.jhotdraw.standard.StandardDrawing+.figureInvalidated(
                org.jhotdraw.framework.FigureChangeEvent)) &&
        this(object) && args(event);

    pointcut fireDrawingTitleChanged(
            org.jhotdraw.standard.StandardDrawing object) :
        execution (* org.jhotdraw.standard.StandardDrawing+.fireDrawingTitleChanged()) &&
        this(object);

    pointcut figureRequestUpdate(org.jhotdraw.framework.FigureChangeEvent event,
            org.jhotdraw.standard.StandardDrawing object) :
        execution (* org.jhotdraw.standard.StandardDrawing+.figureRequestUpdate(
                org.jhotdraw.framework.FigureChangeEvent)) &&
        this(object) && args(event);

    //--------------------------------------------------------------------
    //ITDs
    //optimization: store event in the object, instead of forcing the tracematch
    //to match it. Much, much faster. Can be broken, though, if FigureChangeEvents
    //can be nested. The correct behavior would have been to store the FigureChangeEvent
    //in the listeners 
    private FigureChangeEvent StandardDrawing.event;
    
    public void StandardDrawing.setFigureChangeEvent(FigureChangeEvent event) {
        this.event=event;
    }
    
    public FigureChangeEvent StandardDrawing.getFigureChangeEvent() {
        return this.event;
    }
    
    before(FigureChangeEvent event, StandardDrawing object) : 
        figureInvalidated(event, object) {
        object.setFigureChangeEvent(event);
    }
    
    
    //--------------------------------------------------------------------
    //Tracematches
    tracematch (DrawingChangeListener listener, 
            StandardDrawing object/*,
            FigureChangeEvent event*/) {
        sym addListener after : addDrawListener(listener, object);
    	sym removeListener after : removeDrawListener(listener, object);
    	sym figureInvalidated after : 
    	    figureInvalidated(org.jhotdraw.framework.FigureChangeEvent,
    	            /*event,*/ object);
    
    	addListener figureInvalidated+ {
    	    /*debPrint("Tracematch: figure invalidated. " +
    	            "Event: " + object.getFigureChangeEvent().toString() + 
    	            "Subject: " + object.toString() +
    	            "Listener: " + listener.toString());*/
    	    //moved from figureInvalidated
    	    listener.drawingInvalidated(
    	            new DrawingChangeEvent(object, 
    	                    object.getFigureChangeEvent().getInvalidatedRectangle())
    	            );
    	}
    }    
    //would have been nice if I could have put all the trace match actions
    //in one tracematch, as the symbols addListener and removeListener are
    //common to these tracematches...
	tracematch (DrawingChangeListener listener, 
            StandardDrawing object) {
        sym addListener after : addDrawListener(listener, object);
    	sym removeListener after : removeDrawListener(listener, object);
    	sym fireDrawingTitleChanged after : fireDrawingTitleChanged(object);
    
    	addListener fireDrawingTitleChanged+ {
    	    debPrint("Tracematch: title changed " +
    	            "Subject: " + object.toString() +
    	            "Listener: " + listener.toString());
            listener.drawingTitleChanged(
                    new DrawingChangeEvent(
                            object, 
                            null)
                     );
    	}
    }

    tracematch (DrawingChangeListener listener, 
            StandardDrawing object) {
        sym addListener after : addDrawListener(listener, object);
    	sym removeListener after : removeDrawListener(listener, object);
    	sym figureRequestUpdate after : figureRequestUpdate(org.jhotdraw.framework.FigureChangeEvent, object);
    
    	addListener figureRequestUpdate+ {
    	    debPrint("Tracematch: figure request update. " +
    	            "Subject: " + object.toString() +
    	            "Listener: " + listener.toString());
    	    //moved from figureRequestUpdate
    	    listener.drawingRequestUpdate(new DrawingChangeEvent(object, null));    	
    	}
    }    
}