import org.aspectj.lang.*;
import org.aspectj.lang.reflect.*;

/*
 * Similar to Tracematch, but checks if the NPE occured on the same line
 * as the field-get. 
 */
public aspect Tracematch2 {

  pointcut anyset(Object t, Object o) :
   set(* *) && args(o) && target(t) && !within(TraceMatch2);

  pointcut nullset(Object t,Object o) :
   anyset(t,o) && if(o==null);

  pointcut anyget(Object t) :
    get(* *) && target(t) && 
    !within(TraceMatch2);
 
  // the infamous let is needed here
  pointcut signature(String s) :
    let (s, thisJoinPoint.getSignature().toString().intern());

  pointcut lineno(int n) :
      let (n, thisJoinPoint.getSourceLocation().getLine());

  tracematch(Object t, String s,
             NullPointerException ne, SourceLocation sloc,int n)
  {
    sym setnull before : nullset(t,*) && signature(s) &&
                         let(sloc,thisJoinPoint.getSourceLocation()) ;
    sym setany before : nullset(t,*) && signature(s);
    sym anyget after returning : anyget(t) && signature(s) && lineno(n);
    sym npe after throwing(ne) : (call(* *.*(..)) || get(* *) || set(* *)) &&
	                         lineno(n);

    setnull anyget npe

   {
      System.out.println("Null pointer exception at "
                         + thisJoinPoint.getSourceLocation());
      System.out.println("   possibly caused by null value of "+s);
      System.out.println("   which was set to null at "+sloc);
   }

  }
  
}

class NullPointerCheckTest2 {
    
    static class Container {
        String s = "aString";
    }
    
    public static void main(String[] args) {
        Container foo = new Container();
        foo.s = null;
        System.out.println("Tracematch should now be triggered");
        foo.s.toLowerCase();
    }
}
    
