import org.aspectj.lang.*;
import org.aspectj.lang.reflect.*;
import java.util.*;

public aspect Aspect {

    pointcut inaspect() :
               within(Aspect) 
            || within(IdentityPair) 
            || within(org.aspectj..*)
            || within(org.aspectbench..*) 
            || cflow(execution(* Aspect.*(..)))
            || cflow(within(Aspect) && adviceexecution());

    pointcut weave() :
        execution(* abc.weaving.weaver.Weaver.weaveAdvice());
    
    pointcut unweave() :
        execution(* abc.weaving.weaver.Weaver.resetForReweaving());

    private Set notYetUnwoven = new HashSet();

    before(Object o): set(* *) && target(o) && cflow(weave()) && !inaspect() {
        if(weavingIteration > 1) return;
        String f = thisJoinPoint.getSignature().toString().intern();
        notYetUnwoven.add(new IdentityPair(o, f));
    }
    before(Object o): set(* *) && target(o) && cflow(unweave()) && !inaspect() {
        String f = thisJoinPoint.getSignature().toString().intern();
        notYetUnwoven.remove(new IdentityPair(o, f));
    }

    private int weavingIteration = 0;
    before(): weave() && !inaspect() {
        weavingIteration++;
    }

    before(Object o): get(* *) && target(o) && !inaspect() {
        if(weavingIteration < 2) return;
        String f = thisJoinPoint.getSignature().toString().intern();
        if(notYetUnwoven.contains(new IdentityPair(o, f))) {
            System.out.println("Forgot to unweave field "+f+
                    " of an object of type "+o.getClass());
        }
        notYetUnwoven.remove(new IdentityPair(o, f));
    }
}

class IdentityPair {
    private Object o;
    private String s;
    public IdentityPair(Object o, String s) {
        this.o = o;
        this.s = s;
    }
    public int hashCode() {
        // return o.hashCode()+s.hashCode();
        // We can't call o.hashCode() because it's likely to have
        // side-effects (and gets which we're going to intercept)
        return 0;
    }
    public boolean equals(Object other) {
        IdentityPair oth = (IdentityPair) other;
        if(o != oth.o) return false;
        if(s != oth.s) return false;
        return true;
    }
}
